@@ -20,6 +20,7 @@ import signal  # for file operations, to kill processes, for regex, for subproce
 import subprocess
 import tempfile
 import os
+import socket
 from PyQt6.QtCore import QTimer, QElapsedTimer, QVariant
 
 from app.ApplicationInfo import applicationInfo
@@ -48,6 +49,17 @@ def normalize_path(path):
     return os.path.normpath(path).replace("\\", "/")
 
 class Controller:
+    @staticmethod
+    def _has_ipv6_connectivity():
+        try:
+            sock = socket.socket(socket.AF_INET6, socket.SOCK_DGRAM)
+            sock.settimeout(1)
+            sock.connect(("2606:4700:4700::1111", 53))
+            sock.close()
+            return True
+        except OSError as exc:
+            log.debug(f"IPv6 connectivity probe failed: {exc}")
+            return False
 
     # initialisations that will happen once - when the program is launched
     @timing
@@ -218,6 +230,14 @@ class Controller:
         self.start(os.path.basename(self.logic.activeProject.properties.projectName))
         self.view.restoreToolTabs() # restores the tool tabs for each host
         self.view.hostTableClick() # click on first host to restore his host tool tabs
+        try:
+            repo_container = getattr(self.logic.activeProject, "repositoryContainer", None)
+            if repo_container and hasattr(repo_container, "processRepository"):
+                repo_container.processRepository.resetDisplayStatusForOpenProcesses()
+                self.view.refreshToolsTableModel()
+                self.view.viewState.lazy_update_tools = True
+        except Exception:
+            log.exception("Failed to reset process display status when opening project")
 
     def saveProject(self, lastHostIdClicked, notes):
         if not lastHostIdClicked == '':
@@ -233,7 +253,6 @@ class Controller:
         self.saveSettings() # backup and save config file, if necessary
         self.screenshooter.terminate()
         self.initScreenshooter()
-        self.logic.activeProject.repositoryContainer.processRepository.toggleProcessDisplayStatus(True)
         self.view.updateProcessesTableView() # clear process table
         self.logic.projectManager.closeProject(self.logic.activeProject)
 
@@ -242,11 +261,42 @@ class Controller:
         clipboard.setText(data) # Assuming item.text() contains the IP or hostname
 
     @timing
-    def addHosts(self, targetHosts, runHostDiscovery, runStagedNmap, nmapSpeed, scanMode, nmapOptions = []):
+    def addHosts(self, targetHosts, runHostDiscovery, runStagedNmap, nmapSpeed, scanMode,
+                 nmapOptions=None, enableIPv6=False):
         if targetHosts == '':
             log.info('No hosts entered..')
             return
 
+        if nmapOptions is None:
+            nmapOptions = []
+        else:
+            nmapOptions = [opt for opt in nmapOptions if opt]
+
+        # Normalize whitespace
+        nmapOptions = [opt.strip() for opt in nmapOptions if opt and opt.strip()]
+
+        if enableIPv6 and not self._has_ipv6_connectivity():
+            log.warning("IPv6 connectivity test failed. Falling back to IPv4-only scanning.")
+            try:
+                self.view.ui.statusbar.showMessage('IPv6 connectivity unavailable; falling back to IPv4', 5000)
+            except Exception:
+                pass
+            enableIPv6 = False
+
+        incompatible_prefixes = ('-f', '--randomize-hosts', '--data-length')
+        if enableIPv6:
+            filtered_options = []
+            removed = []
+            for opt in nmapOptions:
+                lower = opt.lower()
+                if lower.startswith(incompatible_prefixes):
+                    removed.append(opt)
+                    continue
+                filtered_options.append(opt)
+            if removed:
+                log.info(f"Removing IPv6-incompatible nmap options: {', '.join(removed)}")
+            nmapOptions = filtered_options
+
         import os
         runningFolder = normalize_path(self.logic.activeProject.properties.runningFolder)
         # Use the session directory for temp files
@@ -257,38 +307,66 @@ class Controller:
             session_dir = runningFolder
         # Use the tool output directory directly, not a subdirectory
         tool_output_dir = session_dir
+        ipv6_flag = enableIPv6
+
         if scanMode == 'Easy':
             if runStagedNmap:
-                self.runStagedNmap(targetHosts, runHostDiscovery)
+                self.runStagedNmap(targetHosts, discovery=runHostDiscovery, enable_ipv6=ipv6_flag)
             elif runHostDiscovery:
                 outputfile = normalize_path(os.path.join(tool_output_dir, f"{getTimestamp()}-host-discover"))
-                nmapOptionsString = ' '.join(nmapOptions)
-                command = (
-                    f"nmap {nmapOptionsString} -sV -O --version-light -T{str(nmapSpeed)} "
-                    f"{targetHosts} --stats-every 10s -oA {outputfile}"
-                )
+                easy_mode_flags = ['-f', '--data-length 5', '--randomize-hosts', '--max-retries 2']
+                if ipv6_flag:
+                    removed_easy = [flag for flag in easy_mode_flags if flag.startswith('-f') or flag.startswith('--randomize-hosts') or flag.startswith('--data-length')]
+                    if removed_easy:
+                        log.info(f"Removing IPv6-incompatible easy-mode options: {', '.join(removed_easy)}")
+                    easy_mode_flags = [flag for flag in easy_mode_flags if flag not in removed_easy]
+
+                command_tokens = ["nmap"]
+                if ipv6_flag:
+                    command_tokens.append("-6")
+                command_tokens.extend(nmapOptions)
+                command_tokens.extend(easy_mode_flags)
+                command_tokens.extend([
+                    "-sV", "-O", "--version-light", f"-T{str(nmapSpeed)}",
+                    targetHosts, "--stats-every", "10s", "-oA", outputfile
+                ])
+                command = ' '.join(token for token in command_tokens if token)
                 self.runCommand('nmap', 'nmap (discovery)', targetHosts, '', '', command, getTimestamp(True),
-                                outputfile, self.view.createNewTabForHost(str(targetHosts), 'nmap (discovery)', True))
+                                outputfile, self.view.createNewTabForHost(str(targetHosts), 'nmap (discovery)', True),
+                                enable_ipv6=ipv6_flag)
             else:
                 outputfile = normalize_path(os.path.join(tool_output_dir, f"{getTimestamp()}-nmap-list"))
-                nmapOptionsString = ' '.join(nmapOptions)
-                command = (
-                    "nmap " + nmapOptionsString + " -sL -T" + str(nmapSpeed) + " " + targetHosts +
-                    " --stats-every 10s -oA " + outputfile
-                )
+                command_tokens = ["nmap"]
+                if ipv6_flag:
+                    command_tokens.append("-6")
+                command_tokens.extend(nmapOptions)
+                command_tokens.extend([
+                    "-sL", f"-T{str(nmapSpeed)}", targetHosts, "--stats-every", "10s", "-oA", outputfile
+                ])
+                command = ' '.join(token for token in command_tokens if token)
                 self.runCommand('nmap', 'nmap (list)', targetHosts, '', '', command, getTimestamp(True),
                                 outputfile,
-                                self.view.createNewTabForHost(str(targetHosts), 'nmap (list)', True))
+                                self.view.createNewTabForHost(str(targetHosts), 'nmap (list)', True),
+                                enable_ipv6=ipv6_flag)
         elif scanMode == 'Hard':
             outputfile = normalize_path(os.path.join(tool_output_dir, f"{getTimestamp()}-nmap-custom"))
-            nmapOptionsString = ' '.join(nmapOptions)
-            if 'randomize' not in nmapOptionsString:
-                nmapOptionsString = nmapOptionsString + " -T" + str(nmapSpeed)
-            command = "nmap " + nmapOptionsString + " " + targetHosts + " --stats-every 10s -oA " + outputfile
-            self.runCommand('nmap', 'nmap (custom ' + nmapOptionsString + ')', targetHosts, '', '', command,
+            options_tokens = list(nmapOptions)
+            if not any('randomize' in opt.lower() for opt in options_tokens):
+                options_tokens.append(f"-T{str(nmapSpeed)}")
+            if ipv6_flag and not any(opt.strip().startswith('-6') for opt in options_tokens):
+                options_tokens.insert(0, '-6')
+            options_tokens = [opt for opt in options_tokens if opt]
+            options_str = ' '.join(options_tokens).strip()
+            command_tokens = ["nmap"]
+            command_tokens.extend(options_tokens)
+            command_tokens.extend([targetHosts, "--stats-every", "10s", "-oA", outputfile])
+            command = ' '.join(token for token in command_tokens if token)
+            display_label = options_str
+            self.runCommand('nmap', 'nmap (custom ' + display_label + ')', targetHosts, '', '', command,
                             getTimestamp(True), outputfile,
                             self.view.createNewTabForHost(
-                                str(targetHosts), 'nmap (custom ' + nmapOptionsString + ')', True))
+                                str(targetHosts), 'nmap (custom ' + display_label + ')', True),
+                            enable_ipv6=ipv6_flag)
 
     #################### CONTEXT MENUS ####################
 
@@ -781,6 +859,9 @@ class Controller:
         return self.logic.activeProject.repositoryContainer.processRepository.getProcesses(filters, showProcesses, sort,
                                                                                            ncol)
 
+    def getProcessesForRestore(self):
+        return self.logic.activeProject.repositoryContainer.processRepository.getProcessesForRestore()
+
     #################### PROCESSES ####################
 
     def checkProcessQueue(self):
@@ -860,7 +941,7 @@ class Controller:
 
     # this function creates a new process, runs the command and takes care of displaying the ouput. returns the PID
     # the last 3 parameters are only used when the command is a staged nmap
-    def runCommand(self, *args, discovery=True, stage=0, stop=False):
+    def runCommand(self, *args, discovery=True, stage=0, stop=False, enable_ipv6=False):
         def handleProcStop(*vargs):
             updateElapsed.stop()
             self.processTimers[qProcess.id] = None
@@ -959,8 +1040,16 @@ class Controller:
             log.info(f"runCommand connected for stage {str(stage)}")
             nextStage = stage + 1
             qProcess.finished.connect(
-                lambda: self.runStagedNmap(str(hostIp), discovery=discovery, stage=nextStage,
-                                           stop=processRepository.isKilledProcess(str(qProcess.id))))
+                lambda host=str(hostIp), discovery_flag=discovery, next_stage=nextStage,
+                       enable_ipv6_flag=enable_ipv6, process_id=qProcess.id:
+                self.runStagedNmap(
+                    host,
+                    discovery=discovery_flag,
+                    stage=next_stage,
+                    stop=processRepository.isKilledProcess(str(process_id)),
+                    enable_ipv6=enable_ipv6_flag
+                )
+            )
 
         return getPid(qProcess)  # return the pid so that we can kill the process if needed
 
@@ -995,7 +1084,7 @@ class Controller:
         return getPid(qProcess)
 
     # recursive function used to run nmap in different stages for quick results
-    def runStagedNmap(self, targetHosts, discovery = True, stage = 1, stop = False):
+    def runStagedNmap(self, targetHosts, discovery=True, stage=1, stop=False, enable_ipv6=False):
         import os
         log.info(f"runStagedNmap called for stage {str(stage)}")
         runningFolder = self.logic.activeProject.properties.runningFolder
@@ -1034,19 +1123,29 @@ class Controller:
                 return
 
             if discovery:                                           # is it with/without host discovery?
-                command = "nmap -T4 -sV -sSU -O "
+                command = "nmap "
+                if enable_ipv6:
+                    command += "-6 "
+                command += "-T4 -sV -sSU -O "
             else:
-                command = "nmap -Pn -sSU "
+                command = "nmap "
+                if enable_ipv6:
+                    command += "-6 "
+                command += "-Pn -sSU "
 
             if stageOp == 'PORTS':
                 command += '-p ' + stageOpValues + ' -vvvv ' + targetHosts + ' -oA ' + outputfile
             elif stageOp == 'NSE':
-                command = 'nmap -sV --script=' + stageOpValues + ' -vvvv ' + targetHosts + ' --stats-every 10s -oA ' + outputfile
+                command = 'nmap '
+                if enable_ipv6:
+                    command += '-6 '
+                command += '-sV --script=' + stageOpValues + ' -vvvv ' + targetHosts + ' --stats-every 10s -oA ' + outputfile
 
             log.debug(f"Stage {str(stage)} command: {str(command)}")
 
             self.runCommand('nmap', 'nmap (stage ' + str(stage) + ')', str(targetHosts), '', '', command,
-                            getTimestamp(True), outputfile, textbox, discovery=discovery, stage=stage, stop=stop)
+                            getTimestamp(True), outputfile, textbox, discovery=discovery, stage=stage, stop=stop,
+                            enable_ipv6=enable_ipv6)
 
     def importFinished(self):
         # if nmap import was the first action, we need to hide the overlay (note: we shouldn't need to do this
@@ -1155,6 +1254,12 @@ class Controller:
             except Exception:
                 log.exception(f"Error storing process output for {qProcess.id}")
 
+            try:
+                self.view.refreshToolsTableModel()
+                self.view.viewState.lazy_update_tools = True
+            except Exception:
+                log.exception("Failed to refresh tools table after process completion")
+
             if 'hydra' in qProcess.name:
                 try:
                     self.view.findFinishedBruteTab(
